var Level = {
    
    game_running: false,
    
    collision_objects: [],
    
    boxes: [],
    
    planet_objects: [],
    
    level_container: new PIXI.Container(),
    
    star_field_container: new PIXI.Container(),
    
    collision_explosion: null,
    
    scroll_position_x: 0,
    scroll_position_y: 0,
    scroll_area: 400,
    
    addPlanet: function(planet) {
        this.planet_objects.push(planet);
        this.level_container.addChild(planet.sprite);
        // We can collide with a planet
        this.addCollisionObject(planet);
    },
    
    addBox: function(x,y, points) {
        var box = new Box();
        box.sprite.x = x;
        box.sprite.y = y;
        box.points = points;
        this.boxes.push(box);
        console.log('Adding box');
        this.level_container.addChild(box.sprite);
        this.addCollisionObject(box);
    },
    
    addCollisionObject: function(collision_object) {
        this.collision_objects.push(collision_object);
    },
    
    star_field_texture: PIXI.Texture.from('gfx/star_field.png'),
    
    star_field_1: null,
    star_field_2: null,
    
    prepareStarfields: function() {
        var prev_state = this.game_running;
        this.game_running = false;
        this.star_field_container.removeChildren();
        this.star_field_1 = new PIXI.TilingSprite(this.star_field_texture, $(window).innerWidth(), $(window).innerHeight());
        this.star_field_2 = new PIXI.TilingSprite(this.star_field_texture, $(window).innerWidth(), $(window).innerHeight());
        this.star_field_container.addChild(this.star_field_1);
        this.star_field_container.addChild(this.star_field_2);
        this.star_field_2.tileScale.x = 0.7;
        this.star_field_2.tileScale.y = 0.7;
        this.game_running = prev_state;
    },
    
    set: function(level, stage) {
        // Clear stage
        stage.removeChildren();
        
        // Clear level
        this.level_container.removeChildren();
        this.level_container.setTransform(0,0);

        // Prepare starfields
        this.prepareStarfields();
        stage.addChild(this.star_field_container);

        // Place spaceship
        Spaceship.reset(1000,1000);
        Spaceship.sprite.visible = true;
        this.level_container.addChild(Spaceship.sprite);
        this.center_on_spaceship();
        
        // Clear objects
        this.collision_objects = [];
        this.planet_objects = [];
        
        // Reset score
        Score.reset();
        
        // Clear boxes
        this.boxes = [];
        
        // Ready collision explosion
        this.collision_explosion = new Explosion();
        this.level_container.addChild(this.collision_explosion.sprite);
        
        switch(level) {
            case 1:
                // Planets on level 1
                var planet = new Planet(6, 700, 700);
                this.addPlanet(planet);
                var planet = new Planet(4, 200, 200);
                this.addPlanet(planet);
                var planet = new Planet(5, 1400, 400);
                this.addPlanet(planet);
                var planet = new Planet(7, 2000, 560);
                this.addPlanet(planet);
                // Boxpositions on level 1
                this.boxpositions = [
                    {x: 300, y: 300, p: 300},
                    {x: 500, y: 500, p: 100},
                    {x: 1050, y: 550, p: 100},
                    {x: 1700, y: 475, p: 100},
                    {x: 2500, y: 560, p: 100},
                    {x: 0, y: 0, p: 200},
                    {x: 700, y: 850, p: 300},
                    {x: 1300, y: 500, p: 200},
                    {x: 1000, y: 100, p: 100},
                    {x: 1800, y: 1000, p: 100},
                ];
                break;
        }
        
        // Add all boxes
        this.boxpositions.forEach(function(position) {
            this.addBox(position.x, position.y, position.p);
        }.bind(this));
        
        stage.addChild(this.level_container);
        
        // Add score
        Score.score_text.x = $(window).innerWidth()-250;
        Score.score_text.y = 20;
        stage.addChild(Score.score_text);
        
        this.game_running = true;
    },
    
    simulate: function(delta) {
        if (! this.game_running) return;
        this.collision_detector();
        this.gravity_logic(delta);
        this.level_scroller(delta);
        this.box_pulsate(delta);
    },
    
    gravity_logic: function(delta) {
        this.planet_objects.forEach(function(planet) {
            planet.getGravityPull(Spaceship.sprite.x, Spaceship.sprite.y);
        }.bind(this));
    },
    
    center_on_spaceship: function() {
        var global_position = Spaceship.sprite.getGlobalPosition();
        this.scroll_position_x = global_position.x-$(window).innerWidth()/2;
        this.scroll_position_y = global_position.y-$(window).innerHeight()/2;
        // Move level
        this.level_container.setTransform(-this.scroll_position_x, -this.scroll_position_y);
    },
    
    level_scroller: function(delta) {
        var global_position = Spaceship.sprite.getGlobalPosition();
        
        if (global_position.x < this.scroll_area) {
            // We need to scroll left
            this.scroll_position_x -= this.scroll_area-global_position.x;
        } else if (global_position.x > $(window).innerWidth()-this.scroll_area) {
            // We need to scroll right
            this.scroll_position_x += this.scroll_area-($(window).innerWidth()-global_position.x);
        }
        
        if (global_position.y < this.scroll_area) {
            // We need to scroll left
            this.scroll_position_y -= this.scroll_area-global_position.y;
        } else if (global_position.y > $(window).innerHeight()-this.scroll_area) {
            // We need to scroll right
            this.scroll_position_y += this.scroll_area-($(window).innerHeight()-global_position.y);
        }
        
        // Move level
        this.level_container.setTransform(-this.scroll_position_x, -this.scroll_position_y);
        
        this.star_field_1.tilePosition.x = -this.scroll_position_x/1.5;
        this.star_field_1.tilePosition.y = -this.scroll_position_y/1.5;
        this.star_field_2.tilePosition.x = -this.scroll_position_x/2;
        this.star_field_2.tilePosition.y = -this.scroll_position_y/2;        
    },
    
    box_pulsate: function(delta) {
        this.boxes.forEach(function(box) {
            box.pulsate(delta);
        });
    },
    
    box_pickup: function(box) {
        // Find box in array
        var i = this.boxes.indexOf(box);
        if (i > -1) this.boxes.splice(i,1);
        var i = this.collision_objects.indexOf(box);
        if (i > -1) this.collision_objects.splice(i,1);
        // Score it
        Score.addScore(box.points);
        box.sprite.destroy();
    },
        
    
    collision_detector: function() {
        this.collision_objects.forEach(function(v) {
            var dist = distance(Spaceship.sprite.x, Spaceship.sprite.y, v.sprite.x, v.sprite.y);
            if (dist < v.diameter/2+25) {
                // We collided. Check what we collided with?
                if (v.type == 'box') {
                    this.box_pickup(v);
                    return;
                }
                this.gameover();
                return;
            }
        }.bind(this));
    },
    
    gameover: function() {
        this.game_running = false;
        Spaceship.sprite.visible = false;
        this.collision_explosion.explode(Spaceship.sprite.x, Spaceship.sprite.y);
        game.stage.addChild(BigCenter.text_object);
        BigCenter.center();
        BigCenter.fadeIn('GAME OVER');
    }
    
}